/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knetproc.h"
#include "settings.h"
#include "statgraph.h"
#include "statpopup.h"
#include "loadlocal.h"

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef HAVE_KSNMP
# include "loadremote.h"
#endif

#include <klocale.h>
#include <kdebug.h>

#ifdef HAVE_STDINT_H
# include <stdint.h>
#elif defined HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

KNetProc::KNetProc(QWidget *parent, uint index, const char *name) :
	QObject(parent, name), m_sets(NULL)
{
	m_popup = new StatPopup(parent, "stat_popup");

	m_sets = new Settings(QString::number(index));
	m_sets->readConfig();
	
	if ( m_sets->type() == Settings::EnumType::Local )
		m_reader = new LoadLocal(this, m_sets->name(), "reader");
	else
#ifdef HAVE_KSNMP
		m_reader = new LoadRemote(this, m_sets->server(), m_sets->community(), m_sets->name().toUInt(), "reader");
#else
	{
		kdError() << "Requested monitor of remote interface without libksnmp." << endl;
		m_reader = NULL;
	}
#endif

	m_in = new StatGraph(parent, 1, 0, "IN", StatGraph::defaultBgColor, "labelIn");
	connect(m_in, SIGNAL(clickedLeft()), this, SLOT(togglePopup()));
	m_out = new StatGraph(parent, 1, 0, "OUT", StatGraph::defaultBgColor, "labelOut");
	connect(m_out, SIGNAL(clickedLeft()), this, SLOT(togglePopup()));
	
	reloadSettings();
}

KNetProc::~KNetProc()
{
	delete m_in;
	delete m_out;
}

void KNetProc::togglePopup()
{
	if ( m_popup->isShown() )
		m_popup->hide();
	else
		m_popup->show();
}

void KNetProc::reloadSettings()
{
	m_sets->readConfig();
	
	m_in->setReadingColor(m_sets->color1());
	m_in->setStyle((StatGraph::Style)m_sets->style());
	m_in->setGridPace( m_sets->grid() ? 6 : 0 );
	m_in->setBgColor( m_sets->transparentBackground() ? StatGraph::invalidColor : m_sets->colorBackground() );
	m_in->setLabel( m_sets->labelled() ? "IN" : QString::null );
	m_in->update();

	m_out->setReadingColor(m_sets->color2());
	m_out->setStyle((StatGraph::Style)m_sets->style());
	m_out->setGridPace( m_sets->grid() ? 6 : 0 );
	m_out->setBgColor( m_sets->transparentBackground() ? StatGraph::invalidColor : m_sets->colorBackground() );
	m_out->setLabel( m_sets->labelled() ? "OUT" : QString::null );
	m_out->update();
	
	if ( ! m_sets->enabled() )
		m_popup->setText(i18n("Disabled."));
}

void KNetProc::takeReading()
{
	if ( ! m_sets->enabled() ) return;
	
	if ( ! m_reader ) return;
	
	if ( ! m_reader->doReading() ) return;

	float bitRateIn = m_reader->recentBytesIn() * 8.0 / (m_sets->timeout() / 1000.0);
	float bitRateOut = m_reader->recentBytesOut() * 8.0 / (m_sets->timeout() / 1000.0);

	kdDebug() << "bitRateIn " << bitRateIn << endl
		<< "bitRateOut " << bitRateOut << endl;
	
	// Convert to percentages.
	int8_t upperIn = (int8_t) (100.0 * bitRateIn / m_sets->scaleIn());
	if (upperIn < 0) upperIn = 0;
	else if (upperIn > 100) upperIn = 100;
	
	int8_t upperOut = (int8_t) (100.0 * bitRateOut /
	    (m_sets->scaleOut() ? m_sets->scaleOut() : m_sets->scaleIn() ) );
	if (upperOut < 0)
		upperOut = 0;
	else if (upperOut > 100)
		upperOut = 100;
	
	m_in->addPercentReading(upperIn, m_sets->soft());
	m_out->addPercentReading(upperOut, m_sets->soft());

	if ( !m_popup->isVisible() ) return;
	
	if ( m_sets->speedUnit() == 0 ) // bits per second
	{
		float currBitIn = (int)((bitRateIn + 50) / 100) / 10.0;
		float currBitOut = (int)((bitRateOut + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Interface: %1\n"
			"Current In: %2 Kb/s, Total In: %3 MB.\n"
			"Current Out: %4 Kb/s, Total Out: %5 MB.")
			.arg(m_sets->name())
			.arg(currBitIn).arg(m_reader->totalMbIn())
			.arg(currBitOut).arg(m_reader->totalMbOut()));
	} else { // bytes per second
		float currByteIn = (int)((bitRateIn/8 + 50) / 100) / 10.0;
		float currByteOut = (int)((bitRateOut/8 + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Interface: %1\n"
			"Current In: %2 KB/s, Total In: %3 MB.\n"
			"Current Out: %4 KB/s, Total Out: %5 MB.")
			.arg(m_sets->name())
			.arg(currByteIn).arg(m_reader->totalMbIn())
			.arg(currByteOut).arg(m_reader->totalMbOut()));
	}
	m_popup->resizeToText();
}

#include "knetproc.moc"
