/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "loadlocal.h"

#include <kdebug.h>
#include <qdir.h>
#include <qregexp.h>

#include <kiconloader.h>
#include <kglobal.h>

// BSD-specific includes.
#ifdef Q_OS_BSD4
# include <stdlib.h>
# include <sys/param.h>
# include <sys/sysctl.h>
# include <sys/socket.h>
# include <net/if_dl.h>
# include <net/route.h>
# include <net/if.h>
#endif

// Solaris-specific includes.
#ifdef Q_OS_SOLARIS
# include <kstat.h>
# nclude <sys/sockio.h>
#endif

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef HAVE_STDINT_H
# include <stdint.h>
#elif defined HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#define MAX_NET_DEV_LINE 512

QStringList LoadLocal::s_interfaces;

LoadLocal::LoadLocal(QObject* parent, const QString &devname, const char* name):
	LoadReader(parent, name), m_device(devname)
{
// OS-specific initialisation.
#ifdef Q_OS_BSD4
	buf = 0;
	alloc = 0;
#endif

#ifdef Q_OS_LINUX
	rx.setName("/sys/class/net/" + m_device + "/statistics/rx_bytes");
	tx.setName("/sys/class/net/" + m_device + "/statistics/tx_bytes");
	useSys = QDir::home().exists("/sys/class/net");
#endif
}

LoadLocal::~LoadLocal()
{
#ifdef Q_OS_BSD4
	free(buf);
#endif
}

const QStringList &LoadLocal::loadInterfaces(bool force)
{
	kdDebug() << "KNetProc::loadInterface(" << force << ")" << endl;
	if ( ! force && ! s_interfaces.empty() )
		return s_interfaces;
	
	if ( force )
		s_interfaces.clear();

#ifndef Q_OS_LINUX
	s_interfaces.push_back("lo");
	s_interfaces.push_back("eth0");
	s_interfaces.push_back("ppp0");
	s_interfaces.push_back("ippp0");
#else
	if ( QDir::root().exists("/sys/class/net") )
	{ // Exists /sys, 2.6 series kernel
		QDir sys("/sys/class/net");
		QStringList l = sys.entryList();
		QStringList::const_iterator end = l.end();
		for(QStringList::const_iterator it = l.begin(); it != end; it++)
		{
			if ( (*it)[0] == '.' ) continue;
			s_interfaces.push_back(*it);
		}
	} else { // Doesn't exists, kernel 2.4 or earlier
		static QFile proc("/proc/net/dev");
		static QRegExp procre("^\\s+(\\S+):(\\d+\\s*){17}$");
		QString read;
		
		if ( ! proc.open(IO_ReadOnly) )
			return s_interfaces;
		
		while ( ! proc.atEnd() )
		{
			if ( proc.readLine(read, MAX_NET_DEV_LINE) == -1 )
			{
				proc.close();
				return s_interfaces;
			}
			
			if ( procre.search(read) == -1 )
				continue;
			
			s_interfaces.push_back(procre.cap(1));
		}
		
		proc.close();
	}
#endif
	return s_interfaces;
}

#ifdef Q_OS_LINUX
bool LoadLocal::doReadingActual()
{
	kdDebug() << "KNetProc::readLoadActual() [Linux]" << endl;
	if ( useSys )
		return readLoadSys();
	else
		return readLoadProc();
}

inline bool LoadLocal::readLoadSys()
{
	kdDebug() << "KNetProc::readLoadSys() [Linux]" << endl;
	QString read;
	
	if ( ! rx.open(IO_ReadOnly) )
		return false;
	
	if ( rx.readLine(read, 32) == -1 )
	{
		rx.close();
		return false;
	}
	
	read.stripWhiteSpace();
	currIn = read.toULong();
	
	rx.close();
	
	if ( ! tx.open(IO_ReadOnly) )
		return false;
	
	if ( tx.readLine(read, 32) == -1 )
	{
		tx.close();
		return false;
	}
	
	read.stripWhiteSpace();
	currOut = read.toULong();
	
	tx.close();

	return true;
}

inline bool LoadLocal::readLoadProc()
{
	kdDebug() << "KNetProc::readLoadProc() [Linux]" << endl;
	static QFile proc("/proc/net/dev");
	static QRegExp procre("^\\s+(\\S+):(\\d+)\\s+(\\d+\\s+){7}(\\d+\\s+)(\\d+\\s*){7}$");
	QString read;
	
	if ( ! proc.open(IO_ReadOnly) )
		return false;
	
	while ( ! proc.atEnd() )
	{
		if ( proc.readLine(read, MAX_NET_DEV_LINE) == -1 )
		{
			proc.close();
			return false;
		}
		
		if ( procre.search(read) == -1 )
			continue;
		
		if ( procre.cap(1) != m_device )
			continue;
		
		currIn = procre.cap(2).toULong();
		currOut = procre.cap(4).toULong();
		break;
	}
	
	proc.close();
	return true;
}
#endif

#ifdef Q_OS_BSD4
bool LoadLocal::doReadingActual()
{
	kdDebug() << "KNetProc::readLoadActual() [BSD4]" << endl;
	static struct if_msghdr *ifm, *nextifm;
	static struct sockaddr_dl *sdl;
	static char *lim, *next;
	static size_t needed;
	static char s[32];
	static int mib[] = { CTL_NET, PF_ROUTE, 0, 0, NET_RT_IFLIST, 0 };
	if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
		return false;
	if (alloc < needed) {
		free(buf);
		buf = (char *) malloc(needed);
		if ( ! buf )
			return false;
		alloc = needed;
	}

	if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0)
		return false;
	lim = buf + needed;

	next = buf;
	while (next < lim)
	{
		ifm = (struct if_msghdr *) next;
		if (ifm->ifm_type != RTM_IFINFO)
			return false;
		next += ifm->ifm_msglen;

		while (next < lim)
		{
			nextifm = (struct if_msghdr *) next;
			if (nextifm->ifm_type != RTM_NEWADDR)
				break;
			next += nextifm->ifm_msglen;
		}

		if (ifm->ifm_flags & IFF_UP)
		{
			sdl = (struct sockaddr_dl *) (ifm + 1);
			if (sdl->sdl_family != AF_LINK)
				continue;
			strncpy(s, sdl->sdl_data, sdl->sdl_nlen);
			s[sdl->sdl_nlen] = 0;
		
			if (m_device == s)
			{
				currIn = ifm->ifm_data.ifi_ibytes;
				currOut = ifm->ifm_data.ifi_obytes;
				return true;
			}
		}
	}
	
	return false;
}
#endif

#ifdef Q_OS_SOLARIS
bool LoadLocal::doReadingActual()
{
	kdDebug() << "KNetProc::readLoadActual() [Solaris]" << endl;
	static kstat_ctl_t *kc;
	static kstat_t *ksp;
	static kstat_named_t *knp;
	kc = kstat_open();
	ksp = kstat_lookup(kc, 0, -1, m_device);
	if (ksp && kstat_read(kc, ksp, 0) >= 0)
	{
		if ((knp = (kstat_named_t *)
			kstat_data_lookup(ksp, "rbytes")))
		currIn = knp->value.ui32;
		if ((knp = (kstat_named_t *)
			kstat_data_lookup(ksp, "obytes")))
		currOut = knp->value.ui32;
		
		kstat_close(kc);
		return true;
	}
	kstat_close(kc);
	return false;
}
#endif

#include "loadlocal.moc"
