/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOADLOCAL_H
#define LOADLOCAL_H

#include <loadreader.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qfile.h>

/**
 * @brief Class for local interfaces stat reading.
 * @author Diego 'Flameeyes' Pettenò
 *
 * This class provides load readings for local interfaces, and allow to get
 * the list of the interfaces present in the system (only for Linux, BSD and
 * Solaris support will come in future).
 * 
 * @todo Add missing support for BSD and Solaris systems to read the
 * interfaces actually in the system.
 *
 * Linux
 * -----
 *
 * For kernels 2.4 and earlier, we use /proc/net/dev file, which contains
 * lines structured in this way:
 * 
 *   iface: bytesIn x x x x x x x bytesOut x x x x x x x
 * 
 * The first two lines are headers, but also if they are not, there are
 * no problems: the regular expression make simpler to parse it.
 * 
 * For kernels 2.5 and greater, if /sys directory is present, it's used
 * to get the statistics.
 *
 * BSD
 * ---
 *
 * BSD code by Roland Riegel <feedback@roland-riegel.de>,
 * Bill Wilson <bill@gkrellm.net>, Hajimu Umemoto <ume@mahoroba.org> and
 * Anthony Mallet <anthony.mallet@useless-ficus.net>.
 *
 * Solaris
 * -------
 *
 * Solaris code by Roland Riegel <feedback@roland-riegel.de>,
 * Bill Wilson <bill@gkrellm.net> and Daisuke Yabuki <dxy@acm.org>.
 *
 * Other Operating Systems
 * -----------------------
 *
 * Please, send in a patch!
 *
 * The KNetLoad maintainer is currently Diego Pettenò
 * <flameeyes@users.berlios.de>, or you could submit a patch through the KDE
 * bug tracking system ( http://bugs.kde.org/ ).
 */
class LoadLocal : public LoadReader
{
Q_OBJECT
public:
	LoadLocal(QObject* parent, const QString &devname, const char* name);
	~LoadLocal();

	/**
	 * @brief Loads the list of interfaces, and return them
	 * @param force Force reload also if m_interfaces is empty
	 * @todo On operating systems different from Linux it fills it with
	 *       stock devices which can not be present in the system.
	 */
	static const QStringList &loadInterfaces(bool force = false);
	
protected:
	virtual bool doReadingActual();
	
	/// Name of the device to read the load of
	QString m_device;

	/// List of all the interfaces found in the system
	static QStringList s_interfaces;

//@{
/**
 * @name osspec OS-specific data members.
 */
#ifdef Q_OS_BSD4
	char* buf;
		///< A temporary buffer whose size may need to change.
	size_t alloc;
		///< The number of bytes currently allocated for buf.
#endif
#ifdef Q_OS_LINUX
	bool useSys;
		///< If true, the /sys interface will be used.
	inline bool readLoadProc();
		///< Read the load from the old /proc interface (2.4 and earlier).
	inline bool readLoadSys();
		///< Read the load from the new /sys interface (since 2.5).
	
	QFile rx;
		///< File object for /sys receiving load.
	QFile tx;
		///< File object for /proc transmitted load.
#endif
//@}
};

#endif
