/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "netloadapplet.h"
#include "settings.h"
#include "interfacesconfigimpl.h"
#include "visualization.h"
#include "knetproc.h"
#include "statgraph.h"
#include "statpopup.h"
#include "interfaceslist.h"
#include "loadlocal.h"

#include <kaboutapplication.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qregexp.h>

#ifdef HAVE_CONFIG_H
# include <config.h>
# undef VERSION // Undefine VERSION in config.h
#endif

#include "../version.h"

#ifdef HAVE_STDINT_H
# include <stdint.h>
#elif defined HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_KSNMP
# include <libksnmp/session.h>
#endif

NetLoadApplet::NetLoadApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
 : KPanelApplet(configFile, type, actions, parent, name)
{
#ifdef HAVE_KSNMP
	KSNMP::init("knetloadapplet");
#endif
	
	KAboutData aboutData(
		"knetloadapplet",
		I18N_NOOP("KNetLoad Applet"),
		VERSION,
		I18N_NOOP("A Kicker applet of KNetLoad"),
		KAboutData::License_GPL,
		"(c) 1999-2000, Markus Gustavsson\n"
		"(c) 2002, Ben Burton\n"
		"(c) 2004-2005, Diego Pettenò",
		I18N_NOOP("Code took from old KNetLoad standalone application"),
		"http://extragear.kde.org/apps/knetload/");

	aboutData.addAuthor("Markus Gustavsson", I18N_NOOP("Original author"), "mighty@fragzone.se");
	aboutData.addAuthor("Ben Burton", I18N_NOOP("KDE3 rewrite"), "bab@debian.org");
	aboutData.addAuthor("Diego Pettenò", I18N_NOOP("2.x+ Versions, current maintainer"), "flameeyes@users.berlios.de");
	aboutData.addAuthor("Roland Riegel", I18N_NOOP("BSD and Solaris support"), "feedback@roland-riegel.de");
	aboutData.addAuthor("Bill Wilson", I18N_NOOP("BSD and Solaris support"), "bill@gkrellm.net");
	aboutData.addAuthor("Hajimu Umemoto", I18N_NOOP("BSD support"), "ume@mahoroba.org");
	aboutData.addAuthor("Anthony Mallet", I18N_NOOP("BSD support"), "anthony.mallet@useless-ficus.net");
	aboutData.addAuthor("Daisuke Yabuki", I18N_NOOP("Solaris support"), "dxy@acm.org");
	
	aboutData.addCredit("Njaard, Charles Samuels", I18N_NOOP("Support and help with KDE"), "charles@kde.org");
	aboutData.addCredit("rikkus, Rik Hemsley", I18N_NOOP("Support and help with KDE"), "rik@kde.org");
	aboutData.addCredit("Mark Halpaap", I18N_NOOP("Independent port to KDE3"), "mark.halpaap@gmx.net");
	aboutData.addCredit(I18N_NOOP("Thanks to everyone in #KDE OpenProjects for being supportive and helpful!"));
	aboutData.addCredit(I18N_NOOP("Thanks obviously to all the creators of, maintainers of, and contributors to the original KNetLoad Application."));
	m_aboutbox = new KAboutApplication(&aboutData, this, "aboutbox");
	
	m_sets = new Settings("");
	m_sets->readConfig();

	m_readTimer = new QTimer(this);
	m_cfgdialog = new KConfigDialog(this, "settings", m_sets);
	
	VisualizationConfig *viscfg = new VisualizationConfig(m_cfgdialog, "Visualization");
	m_cfgdialog->addPage(viscfg, i18n("Visualization"), "style");
	
	InterfacesList *iflist = new InterfacesList(m_cfgdialog, "InterfacesList");
	connect(iflist->btnAdd, SIGNAL(clicked()), this, SLOT(interfaceAdd()));
	connect(iflist->btnRemove, SIGNAL(clicked()), this, SLOT(interfaceRemove()));
	connect(iflist->btnEdit, SIGNAL(clicked()), this, SLOT(interfaceEdit()));
	m_cfgdialog->addPage(iflist, i18n("Interfaces"), "network");
	m_interfaces = iflist->listIFs;
	
	connect(m_cfgdialog, SIGNAL(settingsChanged()), this, SLOT(reloadSettings()));

	connect(m_readTimer, SIGNAL(timeout()), this, SLOT(takeReadings()));
	m_readTimer->start(m_sets->timeout(), true);

	reloadDevices();
}

NetLoadApplet::~NetLoadApplet()
{
	delete m_sets;
	delete m_aboutbox;

#ifdef HAVE_KSNMP
//	KSNMP::shutdown();
#endif
}

void NetLoadApplet::takeReadings()
{
	const QPtrList<KNetProc>::const_iterator end = m_procs.end();
	for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
		(*it)->takeReading();

	m_readTimer->start(m_sets->timeout(), true);
}

void NetLoadApplet::reloadDevices()
{
	{
		const QPtrList<KNetProc>::const_iterator end = m_procs.end();
		for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
		{
			delete *it;
			m_procs.erase(it);
		}
		m_procGroups.clear();
	}
	
	static QRegExp re("interface_(\\d+)");
	
	m_interfaces->clear();
	
	QStringList groups = m_sets->config()->groupList();
	const QStringList::const_iterator end = groups.end();
	for(QStringList::iterator it = groups.begin(); it != groups.end(); it++)
	{
		if ( re.search(*it) == -1 )
			continue;
		
		KNetProc *proc = new KNetProc(this, re.cap(1).toUInt());
		connect(proc->graphIn(), SIGNAL(clickedRight()), this, SLOT(preferences()));
		connect(proc->graphOut(), SIGNAL(clickedRight()), this, SLOT(preferences()));
		// connect(m_readTimer, SIGNAL(timeout()), proc, SLOT(takeReading()));
		m_procs.append(proc);
	
		Settings ifsets(re.cap(1));
		ifsets.readConfig();
		QString title;
		
		if ( ifsets.type() == Settings::EnumType::Local )
		{
			title = i18n("Local %1").arg(ifsets.name());
		} else {
			title = (i18n("Remote %1 @ %2").arg(ifsets.name())).arg(ifsets.server());
		}
		
		m_interfaces->insertItem(title);
		m_procGroups.append(re.cap(1));
	}
	
	emit updateLayout();
}

int NetLoadApplet::widthForHeight(int height) const
{
	if ( m_sets->combinedView() )
	{
		if ( orientation() == Qt::Horizontal )
			return m_procs.count() * ( height/2 + 2 );
		else {
			if ( ! m_procs.count() ) return 0; // Avoid DivBy0 fault
			return (height/m_procs.count()-2)*2;
		}
	} else {
		if ( orientation() == Qt::Horizontal )
			return m_procs.count() * ( height*2 + 4 );
		else {
			if ( ! m_procs.count() ) return 0; // Avoid DivBy0 fault
			return (height/m_procs.count()-4)/2;
		}
	}
}

int NetLoadApplet::heightForWidth(int width) const
{
	if ( m_sets->combinedView() )
	{
		if ( orientation() == Qt::Vertical )
			return m_procs.count() * ( width/2 + 2 );
		else {
			if ( ! m_procs.count() ) return 0; // Avoid DivBy0 fault
			return (width/m_procs.count()-2)*2;
		}
	} else {
		if ( orientation() == Qt::Vertical )
			return m_procs.count() * ( width*2 + 4 );
		else {
			if ( ! m_procs.count() ) return 0; // Avoid DivBy0 fault
			return (width/m_procs.count()-4)/2;
		}
	}
}

void NetLoadApplet::about()
{
	m_aboutbox->show();
}

void NetLoadApplet::preferences()
{
	if(KConfigDialog::showDialog("settings"))
		return;
	
	m_cfgdialog->show();
}

void NetLoadApplet::reloadSettings()
{
	m_sets->writeConfig();
	
	const QPtrList<KNetProc>::const_iterator end = m_procs.end();
	for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
		(*it)->reloadSettings();
}

void NetLoadApplet::resizeEvent(QResizeEvent *e)
{
	KPanelApplet::resizeEvent(e);
	
	uint i = 0;
	const QPtrList<KNetProc>::const_iterator end = m_procs.end();
	for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
	{
		if ( m_sets->combinedView() )
		{
			if ( orientation() == Qt::Horizontal )
			{
				const uint pos = i*(height()/2+2);
				(*it)->graphIn()->setGeometry(pos, 0, height()/2, height()/2);
				(*it)->graphOut()->setGeometry(pos, height()/2+2, height()/2, height()/2);
			} else {
				const uint pos = i*(width()/2+2);
				(*it)->graphIn()->setGeometry(0, pos, width()/2, width()/2);
				(*it)->graphOut()->setGeometry(width()/2+2, pos, width()/2, width()/2);
			}
		} else {
			if ( orientation() == Qt::Horizontal )
			{
				const uint pos = i*(2*height()+4);
				(*it)->graphIn()->setGeometry(pos, 0, height(), height());
				(*it)->graphOut()->setGeometry(pos+height()+2, 0, height(), height());
			} else {
				const uint pos = i*(2*width()+4);
				(*it)->graphIn()->setGeometry(0, pos, width(), width());
				(*it)->graphOut()->setGeometry(0, pos+width()+2, width(), width());
			}
		}
		
		(*it)->graphIn()->show();
		(*it)->graphOut()->show();
		
		i++;
	}
	
	update();
	reloadSettings();
}

void NetLoadApplet::interfaceAdd()
{
	// Here we limit to 256 interfaces, this can change.. but who needs to
	// monitor more than 256 interfaces in a kicker applet? o_O
	uint i;
	for(i = 0; i < 256; i++)
		if( ! m_sets->config()->hasGroup("interface_" + QString::number(i)) )
			break;

	configureIf(QString::number(i));
}

void NetLoadApplet::interfaceRemove()
{
	int itm = m_interfaces->currentItem();
	if ( itm == -1 ) return;

	delete m_procs.at(itm);
	m_procs.remove(itm);
	
	m_sets->config()->deleteGroup("interface_" + *(m_procGroups.at(itm)));
	m_sets->writeConfig();

	reloadDevices();
}

void NetLoadApplet::interfaceEdit()
{
	int itm = m_interfaces->currentItem();
	if ( itm == -1 ) return;
	configureIf(*(m_procGroups.at(itm)));
}

void NetLoadApplet::configureIf(const QString &param)
{
	Settings ifsets(param);
	
	ifsets.readConfig();
	KConfigDialog *ifadd = new KConfigDialog(this, "settings", &ifsets, KDialogBase::Plain);
	InterfacesConfigImpl *ifcfg = new InterfacesConfigImpl(ifadd->plainPage(), "Interfaces");

	ifadd->addPage(ifcfg, i18n("Interface"), "network");
	
	if ( ifadd->exec() == QDialog::Accepted )
	{
		ifsets.writeConfig();
		m_sets->readConfig();
	}
	
	reloadDevices();
	
	delete ifadd;
}

extern "C"
{
	KPanelApplet* KDE_EXPORT init( QWidget *parent, const QString configFile)
	{
		KGlobal::locale()->insertCatalogue("knetloadapplet");
		return new NetLoadApplet(configFile, KPanelApplet::Normal,
			KPanelApplet::About | KPanelApplet::Preferences,
			parent, "knetloadapplet");
	}
}

#include "netloadapplet.moc"
